# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import struct_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.recommendationengine_v1beta1.types import user_event as gcr_user_event

__protobuf__ = proto.module(
    package="google.cloud.recommendationengine.v1beta1",
    manifest={
        "PredictRequest",
        "PredictResponse",
    },
)


class PredictRequest(proto.Message):
    r"""Request message for Predict method.

    Attributes:
        name (str):
            Required. Full resource name of the format:
            ``{name=projects/*/locations/global/catalogs/default_catalog/eventStores/default_event_store/placements/*}``
            The id of the recommendation engine placement. This id is
            used to identify the set of models that will be used to make
            the prediction.

            We currently support three placements with the following IDs
            by default:

            -  ``shopping_cart``: Predicts items frequently bought
               together with one or more catalog items in the same
               shopping session. Commonly displayed after
               ``add-to-cart`` events, on product detail pages, or on
               the shopping cart page.

            -  ``home_page``: Predicts the next product that a user will
               most likely engage with or purchase based on the shopping
               or viewing history of the specified ``userId`` or
               ``visitorId``. For example - Recommendations for you.

            -  ``product_detail``: Predicts the next product that a user
               will most likely engage with or purchase. The prediction
               is based on the shopping or viewing history of the
               specified ``userId`` or ``visitorId`` and its relevance
               to a specified ``CatalogItem``. Typically used on product
               detail pages. For example - More items like this.

            -  ``recently_viewed_default``: Returns up to 75 items
               recently viewed by the specified ``userId`` or
               ``visitorId``, most recent ones first. Returns nothing if
               neither of them has viewed any items yet. For example -
               Recently viewed.

            The full list of available placements can be seen at
            https://console.cloud.google.com/recommendation/datafeeds/default_catalog/dashboard
        user_event (google.cloud.recommendationengine_v1beta1.types.UserEvent):
            Required. Context about the user, what they
            are looking at and what action they took to
            trigger the predict request. Note that this user
            event detail won't be ingested to userEvent
            logs. Thus, a separate userEvent write request
            is required for event logging.
        page_size (int):
            Optional. Maximum number of results to return
            per page. Set this property to the number of
            prediction results required. If zero, the
            service will choose a reasonable default.
        page_token (str):
            Optional. The previous PredictResponse.next_page_token.
        filter (str):
            Optional. Filter for restricting prediction results. Accepts
            values for tags and the ``filterOutOfStockItems`` flag.

            -  Tag expressions. Restricts predictions to items that
               match all of the specified tags. Boolean operators ``OR``
               and ``NOT`` are supported if the expression is enclosed
               in parentheses, and must be separated from the tag values
               by a space. ``-"tagA"`` is also supported and is
               equivalent to ``NOT "tagA"``. Tag values must be double
               quoted UTF-8 encoded strings with a size limit of 1 KiB.

            -  filterOutOfStockItems. Restricts predictions to items
               that do not have a stockState value of OUT_OF_STOCK.

            Examples:

            -  tag=("Red" OR "Blue") tag="New-Arrival" tag=(NOT
               "promotional")
            -  filterOutOfStockItems tag=(-"promotional")
            -  filterOutOfStockItems
        dry_run (bool):
            Optional. Use dryRun mode for this prediction
            query. If set to true, a dummy model will be
            used that returns arbitrary catalog items. Note
            that the dryRun mode should only be used for
            testing the API, or if the model is not ready.
        params (MutableMapping[str, google.protobuf.struct_pb2.Value]):
            Optional. Additional domain specific parameters for the
            predictions.

            Allowed values:

            -  ``returnCatalogItem``: Boolean. If set to true, the
               associated catalogItem object will be returned in the
               ``PredictResponse.PredictionResult.itemMetadata`` object
               in the method response.
            -  ``returnItemScore``: Boolean. If set to true, the
               prediction 'score' corresponding to each returned item
               will be set in the ``metadata`` field in the prediction
               response. The given 'score' indicates the probability of
               an item being clicked/purchased given the user's context
               and history.
        labels (MutableMapping[str, str]):
            Optional. The labels for the predict request.

            -  Label keys can contain lowercase letters, digits and
               hyphens, must start with a letter, and must end with a
               letter or digit.
            -  Non-zero label values can contain lowercase letters,
               digits and hyphens, must start with a letter, and must
               end with a letter or digit.
            -  No more than 64 labels can be associated with a given
               request.

            See https://goo.gl/xmQnxf for more information on and
            examples of labels.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    user_event: gcr_user_event.UserEvent = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gcr_user_event.UserEvent,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=7,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=8,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=3,
    )
    dry_run: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    params: MutableMapping[str, struct_pb2.Value] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=6,
        message=struct_pb2.Value,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=9,
    )


class PredictResponse(proto.Message):
    r"""Response message for predict method.

    Attributes:
        results (MutableSequence[google.cloud.recommendationengine_v1beta1.types.PredictResponse.PredictionResult]):
            A list of recommended items. The order
            represents the ranking (from the most relevant
            item to the least).
        recommendation_token (str):
            A unique recommendation token. This should be
            included in the user event logs resulting from
            this recommendation, which enables accurate
            attribution of recommendation model performance.
        items_missing_in_catalog (MutableSequence[str]):
            IDs of items in the request that were missing
            from the catalog.
        dry_run (bool):
            True if the dryRun property was set in the
            request.
        metadata (MutableMapping[str, google.protobuf.struct_pb2.Value]):
            Additional domain specific prediction
            response metadata.
        next_page_token (str):
            If empty, the list is complete. If nonempty, the token to
            pass to the next request's PredictRequest.page_token.
    """

    class PredictionResult(proto.Message):
        r"""PredictionResult represents the recommendation prediction
        results.

        Attributes:
            id (str):
                ID of the recommended catalog item
            item_metadata (MutableMapping[str, google.protobuf.struct_pb2.Value]):
                Additional item metadata / annotations.

                Possible values:

                -  ``catalogItem``: JSON representation of the catalogItem.
                   Will be set if ``returnCatalogItem`` is set to true in
                   ``PredictRequest.params``.
                -  ``score``: Prediction score in double value. Will be set
                   if ``returnItemScore`` is set to true in
                   ``PredictRequest.params``.
        """

        id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        item_metadata: MutableMapping[str, struct_pb2.Value] = proto.MapField(
            proto.STRING,
            proto.MESSAGE,
            number=2,
            message=struct_pb2.Value,
        )

    @property
    def raw_page(self):
        return self

    results: MutableSequence[PredictionResult] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=PredictionResult,
    )
    recommendation_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    items_missing_in_catalog: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    dry_run: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    metadata: MutableMapping[str, struct_pb2.Value] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=5,
        message=struct_pb2.Value,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=6,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.recommendationengine_v1beta1.types import catalog, user_event

__protobuf__ = proto.module(
    package="google.cloud.recommendationengine.v1beta1",
    manifest={
        "GcsSource",
        "CatalogInlineSource",
        "UserEventInlineSource",
        "ImportErrorsConfig",
        "ImportCatalogItemsRequest",
        "ImportUserEventsRequest",
        "InputConfig",
        "ImportMetadata",
        "ImportCatalogItemsResponse",
        "ImportUserEventsResponse",
        "UserEventImportSummary",
    },
)


class GcsSource(proto.Message):
    r"""Google Cloud Storage location for input content.
    format.

    Attributes:
        input_uris (MutableSequence[str]):
            Required. Google Cloud Storage URIs to input files. URI can
            be up to 2000 characters long. URIs can match the full
            object path (for example,
            ``gs://bucket/directory/object.json``) or a pattern matching
            one or more files, such as ``gs://bucket/directory/*.json``.
            A request can contain at most 100 files, and each file can
            be up to 2 GB. See `Importing catalog
            information </recommendations-ai/docs/upload-catalog>`__ for
            the expected file format and setup instructions.
    """

    input_uris: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=1,
    )


class CatalogInlineSource(proto.Message):
    r"""The inline source for the input config for ImportCatalogItems
    method.

    Attributes:
        catalog_items (MutableSequence[google.cloud.recommendationengine_v1beta1.types.CatalogItem]):
            Optional. A list of catalog items to
            update/create. Recommended max of 10k items.
    """

    catalog_items: MutableSequence[catalog.CatalogItem] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=catalog.CatalogItem,
    )


class UserEventInlineSource(proto.Message):
    r"""The inline source for the input config for ImportUserEvents
    method.

    Attributes:
        user_events (MutableSequence[google.cloud.recommendationengine_v1beta1.types.UserEvent]):
            Optional. A list of user events to import.
            Recommended max of 10k items.
    """

    user_events: MutableSequence[user_event.UserEvent] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=user_event.UserEvent,
    )


class ImportErrorsConfig(proto.Message):
    r"""Configuration of destination for Import related errors.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gcs_prefix (str):
            Google Cloud Storage path for import errors. This must be an
            empty, existing Cloud Storage bucket. Import errors will be
            written to a file in this bucket, one per line, as a
            JSON-encoded ``google.rpc.Status`` message.

            This field is a member of `oneof`_ ``destination``.
    """

    gcs_prefix: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="destination",
    )


class ImportCatalogItemsRequest(proto.Message):
    r"""Request message for Import methods.

    Attributes:
        parent (str):
            Required.
            ``projects/1234/locations/global/catalogs/default_catalog``
        request_id (str):
            Optional. Unique identifier provided by
            client, within the ancestor dataset scope.
            Ensures idempotency and used for request
            deduplication. Server-generated if unspecified.
            Up to 128 characters long. This is returned as
            google.longrunning.Operation.name in the
            response.
        input_config (google.cloud.recommendationengine_v1beta1.types.InputConfig):
            Required. The desired input location of the
            data.
        errors_config (google.cloud.recommendationengine_v1beta1.types.ImportErrorsConfig):
            Optional. The desired location of errors
            incurred during the Import.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    input_config: "InputConfig" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="InputConfig",
    )
    errors_config: "ImportErrorsConfig" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ImportErrorsConfig",
    )


class ImportUserEventsRequest(proto.Message):
    r"""Request message for the ImportUserEvents request.

    Attributes:
        parent (str):
            Required.
            ``projects/1234/locations/global/catalogs/default_catalog/eventStores/default_event_store``
        request_id (str):
            Optional. Unique identifier provided by client, within the
            ancestor dataset scope. Ensures idempotency for expensive
            long running operations. Server-generated if unspecified. Up
            to 128 characters long. This is returned as
            google.longrunning.Operation.name in the response. Note that
            this field must not be set if the desired input config is
            catalog_inline_source.
        input_config (google.cloud.recommendationengine_v1beta1.types.InputConfig):
            Required. The desired input location of the
            data.
        errors_config (google.cloud.recommendationengine_v1beta1.types.ImportErrorsConfig):
            Optional. The desired location of errors
            incurred during the Import.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    input_config: "InputConfig" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="InputConfig",
    )
    errors_config: "ImportErrorsConfig" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="ImportErrorsConfig",
    )


class InputConfig(proto.Message):
    r"""The input config source.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        catalog_inline_source (google.cloud.recommendationengine_v1beta1.types.CatalogInlineSource):
            The Inline source for the input content for
            Catalog items.

            This field is a member of `oneof`_ ``source``.
        gcs_source (google.cloud.recommendationengine_v1beta1.types.GcsSource):
            Google Cloud Storage location for the input
            content.

            This field is a member of `oneof`_ ``source``.
        user_event_inline_source (google.cloud.recommendationengine_v1beta1.types.UserEventInlineSource):
            The Inline source for the input content for
            UserEvents.

            This field is a member of `oneof`_ ``source``.
    """

    catalog_inline_source: "CatalogInlineSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="source",
        message="CatalogInlineSource",
    )
    gcs_source: "GcsSource" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="source",
        message="GcsSource",
    )
    user_event_inline_source: "UserEventInlineSource" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="source",
        message="UserEventInlineSource",
    )


class ImportMetadata(proto.Message):
    r"""Metadata related to the progress of the Import operation.
    This will be returned by the
    google.longrunning.Operation.metadata field.

    Attributes:
        operation_name (str):
            Name of the operation.
        request_id (str):
            Id of the request / operation. This is
            parroting back the requestId that was passed in
            the request.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Operation create time.
        success_count (int):
            Count of entries that were processed
            successfully.
        failure_count (int):
            Count of entries that encountered errors
            while processing.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Operation last update time. If the operation
            is done, this is also the finish time.
    """

    operation_name: str = proto.Field(
        proto.STRING,
        number=5,
    )
    request_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    success_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    failure_count: int = proto.Field(
        proto.INT64,
        number=2,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=6,
        message=timestamp_pb2.Timestamp,
    )


class ImportCatalogItemsResponse(proto.Message):
    r"""Response of the ImportCatalogItemsRequest. If the long
    running operation is done, then this message is returned by the
    google.longrunning.Operations.response field if the operation
    was successful.

    Attributes:
        error_samples (MutableSequence[google.rpc.status_pb2.Status]):
            A sample of errors encountered while
            processing the request.
        errors_config (google.cloud.recommendationengine_v1beta1.types.ImportErrorsConfig):
            Echoes the destination for the complete
            errors in the request if set.
    """

    error_samples: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    errors_config: "ImportErrorsConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ImportErrorsConfig",
    )


class ImportUserEventsResponse(proto.Message):
    r"""Response of the ImportUserEventsRequest. If the long running
    operation was successful, then this message is returned by the
    google.longrunning.Operations.response field if the operation
    was successful.

    Attributes:
        error_samples (MutableSequence[google.rpc.status_pb2.Status]):
            A sample of errors encountered while
            processing the request.
        errors_config (google.cloud.recommendationengine_v1beta1.types.ImportErrorsConfig):
            Echoes the destination for the complete
            errors if this field was set in the request.
        import_summary (google.cloud.recommendationengine_v1beta1.types.UserEventImportSummary):
            Aggregated statistics of user event import
            status.
    """

    error_samples: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    errors_config: "ImportErrorsConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ImportErrorsConfig",
    )
    import_summary: "UserEventImportSummary" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="UserEventImportSummary",
    )


class UserEventImportSummary(proto.Message):
    r"""A summary of import result. The UserEventImportSummary
    summarizes the import status for user events.

    Attributes:
        joined_events_count (int):
            Count of user events imported with complete
            existing catalog information.
        unjoined_events_count (int):
            Count of user events imported, but with
            catalog information not found in the imported
            catalog.
    """

    joined_events_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    unjoined_events_count: int = proto.Field(
        proto.INT64,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
